#-----------------------------------------------------------------------------
#
# Thomas Thomassen
# thomas[at]thomthom[dot]net
#
#-----------------------------------------------------------------------------

# Collection of Entities methods.
#
# @since 2.0.0
module TT::Entities
  
  # Returns a boundingbox for all the given entities.
	#
	# @param [Sketchup::Entities] entities
  #
	# @return [Geom::BoundingBox]
	# @since 2.1.0
  def self.bounds(entities)
    bb = Geom::BoundingBox.new
    for e in entities
      next unless e.respond_to?(:bounds)
      bb.add( e.bounds )
    end
    return bb
  end
  
  
  # Collects the 3d positions of the vertices in the given entities collection.
  # Processes child Groups and Components recursivly.
	#
	# @param [Sketchup::Entities] entities
	# @param [Geom::Transformation] parent_transformation
  #
	# @return [Boolean]
	# @since 2.0.0
  def self.positions(entities, parent_transformation = Geom::Transformation.new)
    pts = []
    vertices = []
    entities.each { |e|
      if e.respond_to?(:vertices)
        vertices << e.vertices
      elsif TT::Instance.is?( e )
        d = TT::Instance.definition(e)
        t = parent_transformation * e.transformation
        sub_pts = self.positions(d.entities, t)
        pts.concat( sub_pts )
      end
    }
    vertices.flatten!
    vertices.uniq!
    pts.concat( vertices.map { |v| v.position.transform( parent_transformation ) } )
    pts
  end
  
end # module TT::Entities