#-----------------------------------------------------------------------------
#
# Thomas Thomassen
# thomas[at]thomthom[dot]net
#
#-----------------------------------------------------------------------------

 # http://juixe.com/techknow/index.php/2007/01/22/ruby-class-tutorial/
 # http://stackoverflow.com/questions/1645398/ruby-include-question
 #
 # @deprecated Alpha stage. Very likely to be subject to change!
module TT::GUI
	
	# @abstract
	# @since 2.4.0
	class Control
		attr_accessor( :ui_id ) # @since 2.4.0
		attr_accessor( :left, :right, :top, :bottom, :height, :width ) # @since 2.4.0
		attr_accessor( :parent, :window ) # @since 2.4.0
		
		# @since 2.4.0
		def initialize(*args)
			@ui_id = 'UI_' + self.object_id.to_s
		end
		
		# @since 2.4.0
		def move(left, top)
			@left = left
			@top  = top
		end
		
		# @since 2.4.0
		def size(width, height)
			@width  = width
			@height = height
		end
		
		# @since 2.4.0
		def position(top, right, bottom, left)
			@top	= top
			@right	= right
			@bottom	= bottom
			@left	= left
		end
		
		# @since 2.4.0
		def css
			props = {
				:left	=> @left,
				:top	=> @top,
				:right	=> @right,
				:bottom	=> @bottom,
				:width	=> @width,
				:height	=> @height
			}
			
			str = ''
			
			props.each { |key, value|
				next if value.nil?
				value = value.to_s + 'px' if value.is_a?(Fixnum)
				str += "#{key}:#{value};"
			}
			
			return str if @left.nil? && @top.nil? && @right.nil? && @bottom.nil?
			return 'position:absolute;' + str
		end
	end

	
	# @abstract +Container+ and +Window+ implements this.
	# @since 2.4.0
	module ContainerElement
		# @since 2.4.0
		attr :child
		
		# @since 2.4.0
		def initialize(*args)
			super(*args)
			@child = []
		end
		
		# @since 2.4.0
		def add_control(control)
			throw :invalid_control unless control.is_a?(Control)
			@child << control
			control.parent = self
			control.window = self.window
		end
		
		# @since 2.4.0
		def to_html
			style = ( (css = self.css).nil? ) ? '' : " style='#{css}'"
			html = "<div class='container' id='#{self.ui_id}' #{style}>\n\n"
			@child.each { |control|
				html += control.to_html
			}
			html += "\n</div>\n"
			return html
		end
	end
	
	
	# @since 2.4.0
	class Container < Control
		include ContainerElement
	end
	
	
	# @since 2.4.0
	class Button < Control
		# @since 2.4.0
		attr_accessor( :caption )
		
		# @since 2.4.0
		def initialize(caption, &on_click)
			super
			@caption = caption
			@on_click = on_click
		end
		
		# @since 2.4.0
		def window=(webdialog)
			webdialog.add_action_callback(self.ui_id) { |dialog, params|
				puts ">> #{self.ui_id}.on_click(#{params})"
				@on_click.call(params)
			}
		end
		
		# @since 2.4.0
		def to_html
			style = ( (css = self.css).nil? ) ? '' : " style='#{css}'"
			html = %{<span class='button normal' id='#{self.ui_id}' #{style} onclick='window.location="skp:#{self.ui_id}"'>#{self.caption}</span>\n}
		end
	end
	
	
	# @since 2.4.0
	class Checkbox < Control
		# @since 2.4.0
		attr_accessor( :checked )
		
		# @since 2.4.0
		def initialize(label)
			super
			@label = label
		end
	end
	
	
	# @since 2.4.0
	class OptionList < Control
	
	end
	
	
	# @since 2.4.0
	class Textbox < Control
		# @since 2.4.0
		attr_accessor( :value )
		
		# @since 2.4.0
		def initialize(value)
			super
			@value = value
		end
		
		# @since 2.4.0
		def to_html
			style = ( (css = self.css).nil? ) ? '' : " style='#{css}'"
			html = "<input type='text' id='#{self.ui_id}' value='#{self.value}' #{style} />\n"
		end
	end
	
	
	# @since 2.4.0
	class Label < Control
		# @since 2.4.0
		attr_accessor( :caption, :align_right )
		
		# @since 2.4.0
		def initialize(caption, align_right=true)
			super
			@caption = caption
			@align_right = align_right
		end
		
		# @since 2.4.0
		def to_html
			style = ( (css = self.css).nil? ) ? '' : " style='#{css}'"
			html = "<label #{style}>#{self.caption}</label>\n"
		end
	end

end # module TT::GUI