#-----------------------------------------------------------------------------
#
# Thomas Thomassen
# thomas[at]thomthom[dot]net
#
#-----------------------------------------------------------------------------

# Collection of Point3d methods.
#
# @since 2.0.0
module TT::Point3d
  
  # Checks if point +c+ is between point +a+ and +b+.
  #
  # Return true if +c+ is on +a+ or +b+.
  #
  # @param [Geom::Point3d] a
  # @param [Geom::Point3d] b
  # @param [Geom::Point3d] c
  # @param [Geom::Point3d] on_point - When +true+, if point +c+ is at the same
  #   position as +a+ or +b+ it is considered to be in between.
  #
  # @since 2.0.0
  def self.between?(a, b, c, on_point = true)
    return false unless c.on_line?([a,b])
    v1 = c.vector_to(a)
    v2 = c.vector_to(b)
    if on_point
      return true  if !v1.valid? || !v2.valid?
    else
      return false if !v1.valid? || !v2.valid?
    end
    !v1.samedirection?(v2)
  end
  
end # module TT::Point3d