/*
 * Common methods for webdialogs.
 *
 * Requires jQuery
 */


/* UI namespace */
var UI = function() {
  return {
  
  
    init : function() {
      // Focus property
      UI.add_focus_property()
      // Buttons
      $('.button').live('mousedown', function() { $(this).addClass('pressed'); return false; });
      $('.button').live('mouseup', function() { $(this).removeClass('pressed'); return false; });
    },
    
    
    /* Toggle the content DIV of the clicked element this function is attached to.
     * Attach this event to the Click event of its sibling;
     * <div>
     *   <h2>Section Header</h2>
     *   <div>Content DIV which is toggled.</div>
     * </div>
     */
    toggle_content : function() {
      $(this).siblings('div').slideToggle('fast');
    },
    
    
    // Ensure links are opened in the default browser.
    redirect_links : function() {
      $('a.url').live('click', function()
      {
        window.location = 'skp:url@' + this.href;
        return false;
      } );
    },
    
    
    /* Loops over all input elements and ensure that they get an .focus class
     * added upon focus and remove it when it loses focus. This is a workaround
     * for IE7's lack of :hover support.
     */
    add_focus_property : function() {
      $('input').live('focusin', function () {
        $(this).addClass('focus');
      });
      $('input').live('focusout', function () {
        $(this).removeClass('focus');
      });
    }
    
    
  };
  
}();
$(document).ready(UI.init);



/*
 * Natural Sort algorithm for Javascript - Version 0.4 - Released under MIT license
 * Author: Jim Palmer (based on chunking idea from Dave Koelle)
 * Contributors: Mike Grier (mgrier.com), Clint Priest, Kyle Adams
 */
function naturalSort(a, b){
  // setup temp-scope variables for comparison evauluation
  var re = /(^[0-9]+\.?[0-9]*[df]?e?[0-9]?$|^0x[0-9a-f]+$|[0-9]+)/gi,
    sre = /(^[ ]*|[ ]*$)/g,
    hre = /^0x[0-9a-f]+$/i,
    ore = /^0/,
    // convert all to strings and trim()
    x = a.toString().replace(sre, '') || '',
    y = b.toString().replace(sre, '') || '',
    // chunk/tokenize
    xN = x.replace(re, '\0$1\0').replace(/\0$/,'').replace(/^\0/,'').split('\0'),
    yN = y.replace(re, '\0$1\0').replace(/\0$/,'').replace(/^\0/,'').split('\0'),
    // hex or date detection
    xD = parseInt(x.match(hre)) || (new Date(x)).getTime(),
    yD = parseInt(y.match(hre)) || xD && (new Date(y)).getTime() || null;
  // natural sorting of hex or dates - prevent '1.2.3' valid date
  if ( y.indexOf('.') < 0 && yD )
    if ( xD < yD ) return -1;
    else if ( xD > yD )	return 1;
  // natural sorting through split numeric strings and default strings
  for(var cLoc=0, numS=Math.max(xN.length, yN.length); cLoc < numS; cLoc++) {
    // find floats not starting with '0', string or 0 if not defined (Clint Priest)
    oFxNcL = !(xN[cLoc] || '').match(ore) && parseFloat(xN[cLoc]) || xN[cLoc] || 0;
    oFyNcL = !(yN[cLoc] || '').match(ore) && parseFloat(yN[cLoc]) || yN[cLoc] || 0;
    // handle numeric vs string comparison - number < string - (Kyle Adams)
    if (isNaN(oFxNcL) !== isNaN(oFyNcL)) return (isNaN(oFxNcL)) ? 1 : -1; 
    // rely on string comparison if different types - i.e. '02' < 2 != '02' < '2'
    else if (typeof oFxNcL !== typeof oFyNcL) {
      oFxNcL += ''; 
      oFyNcL += ''; 
    }
    if (oFxNcL < oFyNcL) return -1;
    if (oFxNcL > oFyNcL) return 1;
  }
  return 0;
}