#-----------------------------------------------------------------------------
#
# Thomas Thomassen
# thomas[at]thomthom[dot]net
#
#-----------------------------------------------------------------------------

require 'TT_Lib2/gui.rb'

# (i) Alpha stage. Very likely to be subject to change!
#
# @example
#   w = TT::GUI::Window.new
#   w.show_window
#
# @since 2.4.0
class TT::GUI::Window < UI::WebDialog
 
	include TT::GUI::ContainerElement
	
	# @since 2.4.0
	attr_accessor( :parent, :window )
	
	# @overload initialize(title, scrollable, pref_key, width, height, left, top, resizable)
	#   @param [optional, String] title
	#   @param [optional, Boolean] scrollable
	#   @param [optional, String] pref_key
	#   @param [optional, Integer] width
	#   @param [optional, Integer] height
	#   @param [optional, Integer] left
	#   @param [optional, Integer] top
	#   @param [optional, Boolean] resizable
	#
	# @todo Allow for optional Hash arguments.
	#
	# @since 2.4.0
	def initialize(*args) 
		# title, scrollable, pref_key, width, height, left, top, resizable
		#   0         1          2       3       4      5    6       7
		#title, scrollable, pref_key, width, height, left, top, resizable = args
		
		@window = self
		
		# Default properties
		@props = {
			:scrollable	=> false,
			:scripts	=> [],
			:styles		=> [],
			:title		=> 'Untitled Window',
		}
		
		# (!) if args[0].is_a?(Hash)
    #if args.length == 1 & args[0].is_a?(Hash)
    #  options.merge!( args[0] )
    #else
    #  title, scroll, key, width, height, left, top,
    #end
		
		# Remember the scrollable value and set this later.
		@props[:scrollable] = args[1] if args.length > 1
		# Ensure the size for fixed windows is set - and not read from the last state.
		self.set_size(args[3], args[4]) unless args[7].nil? || args[7]
		# Turn of the navigation buttons by default.
		self.navigation_buttons_enabled = false if self.respond_to?(:navigation_buttons_enabled)
		
		# Init the real WebDialog
		super(*args)
		
		# Set HTML file
		self.set_file(TT::Lib::path + '/webdialog/window.html')
		
		self.add_action_callback('ready') { |dialog, params|
			puts '>> Dialog Ready'
			#data = TT::JSON.new
			#data[:scrollable] = @scrollable
			#dialog.execute_script("init_window(#{data});")
		}
	end
	
	# @since 2.4.0
	def add_script(file)
		@props[:scripts] << file
	end
	
	# @since 2.4.0
	def add_style(file)
		@props[:styles] << file
	end
	
	# @deprecated Not implemented yet!
	# @since 2.4.0
	def get_text(selector)
		# $(selector).text()
		self.execute_script("Webdialog.get_text('#{selector}');")
		self.get_element_value('RUBY_get_text');
	end
	
	# @deprecated Not implemented yet!
	# @since 2.4.0
	def get_html(selector)
		# $(selector).html()
		self.execute_script("Webdialog.get_html('#{selector}');")
		self.get_element_value('RUBY_get_html');
	end
	
	# @deprecated Not implemented yet!
	# @since 2.4.0
	def get_size
		self.execute_script("Webdialog.get_size();")
		self.get_element_value('RUBY_get_size');
	end
	
	# Open or bring to front the window.
	#
	# @todo Add black callback for ready/load event.
	#
	# @return <nil>
	# @since 2.4.0
	def show_window(modal = false)
		if self.visible?
			self.bring_to_front
		else
			# We use set_html here to prevent Macs loading the whole dialog when the
			# plugin loads. No need to populate the dialog and use extra resources
			# if it will never be used.
			self.set_html(build_html)
			
			if TT::System.is_osx? || modal
				self.show_modal
			else
				self.show
			end
		end
	end

	private
	
	# @since 2.4.0
	def build_html
		html = ''
		path = File.join(TT::Lib::path, 'webdialog') + '/'
		File.open(TT::Lib::path + '/webdialog/window.html', 'r') { |f|
			f.read(nil, html)
		}
		# Insert CSS property to hide scrollbars if they are disabled.
		styles = @props[:styles].map { |f| "<link rel='stylesheet' type='text/css' media='screen' href='#{f}' />" }
		styles << '<style type="text/css">html{overflow:hidden;}</style>' unless @props[:scrollable]
		# Compile strings
		styles  = styles.join("\n\t")
		scripts = @props[:scripts].map { |f| "<script type='text/javascript' src='#{f}'></script>" }.join("\n\t")
		# Insert variables
		html.gsub!('%PATH%', path)
		html.gsub!('%STYLES%', styles)
		html.gsub!('%SCRIPTS%', scripts)
		# Insert HTML from child controls
		content = ''
		@child.each { |control|
			content += control.to_html
		}
		html.gsub!('%CONTENT%', content)
		return html
	end

end # module TT::GUI::Window